/*
 * HFS (aka MAC) filesystem/handler for MorphOS/AmigaOS
 * Author: Marek 'March' Szyprowski <marek@amiga.pl>
 */

#include <string.h>

#include <exec/types.h>
#include <exec/execbase.h>
#include <dos/dosextens.h>
#include <dos/filehandler.h>
#include <devices/input.h>
#include <devices/inputevent.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#ifdef __MORPHOS__
#include <intuition/extensions.h>
#endif

#include <proto/intuition.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "mac_fs.h"
#include "mac_protos.h"

/*-------------------------------------------------------------------------*/  

/* libc wrappers for libhfs */
void *malloc(int size)
{
	return FS_AllocMem(size);
}

void free(void *ptr)
{
	return FS_FreeMem(ptr);
}
 
#ifdef LIBHFSDEBUG
void *stderr;
UBYTE fprintf_buff[256];

#include <stdarg.h>

int	 vsprintf __P((char *, const char *, _BSD_VA_LIST_));

int fprintf(void *stream, char *fmt, ...)
{
	int res;
	va_list args;

	va_start (args, fmt);
	res = vsprintf(fprintf_buff, fmt, args);
	va_end (args);

	kprintf("%s", (LONG)fprintf_buff);
	return res;
}
#endif

/*-------------------------------------------------------------------------*/  

#define MAC_TIME_SHIFT ((365*8+2)*24*60*60)

void MacTime2DateStamp(ULONG date, struct DateStamp *datestamp)
{
	ULONG seconds;
	date -= MAC_TIME_SHIFT;

	datestamp->ds_Days = date/86400;
	seconds = date - (datestamp->ds_Days * 86400);
	datestamp->ds_Minute = (ULONG)((UWORD)(seconds/60));
	datestamp->ds_Tick = (ULONG)((UWORD)(seconds%60)*50);
}


void DateStamp2MacTime(struct DateStamp *datestamp, ULONG *date)
{
	ULONG seconds;

	seconds = datestamp->ds_Tick / 50;
	seconds += datestamp->ds_Minute * 60;
	seconds += datestamp->ds_Days * 86400;
	seconds += MAC_TIME_SHIFT;
	*date = seconds;
}
 

/*-------------------------------------------------------------------------*/

void SendEvent(LONG event)
{
	struct IOStdReq *InputRequest;
	struct MsgPort *InputPort;
	struct InputEvent *ie;

	if ((InputPort = (struct MsgPort*)CreateMsgPort()))
	{
		if ((InputRequest = (struct IOStdReq*)CreateIORequest(InputPort, sizeof(struct IOStdReq))))
		{
			if (!OpenDevice("input.device", 0, (struct IORequest*)InputRequest, 0))
			{
				if ((ie = AllocVec(sizeof(struct InputEvent), MEMF_PUBLIC)))
				{
					ie->ie_NextEvent = 0;
					ie->ie_SubClass = 0;
					ie->ie_Code = 0;
					ie->ie_Qualifier = IEQUALIFIER_MULTIBROADCAST;
					ie->ie_EventAddress = 0;
					CurrentTime(&ie->ie_TimeStamp.tv_secs, &ie->ie_TimeStamp.tv_micro);
					ie->ie_Class = event;
					InputRequest->io_Command = IND_WRITEEVENT;
					InputRequest->io_Data = ie;
					InputRequest->io_Length = sizeof(struct InputEvent);
					DoIO((struct IORequest*)InputRequest);
					FreeVec(ie);
				}
				CloseDevice((struct IORequest*)InputRequest);
			}
			DeleteIORequest (InputRequest);
		}
	    DeleteMsgPort (InputPort);
	}
}

int ErrorReq (STRPTR text, ULONG args[])
{
#undef IntuitionBase
	struct IntuitionBase *IntuitionBase = (void *)glob->IntuitionBase;
	
#ifdef __MORPHOS__
	static const struct TagItem estags[] = {{EES_DiskBased, FALSE}, {TAG_DONE, 0}};

	struct ExtEasyStruct req = {
		sizeof(struct ExtEasyStruct),
		0,
		"MacFilesystem Error",
		text,
		"Ok", /* see REQ_ defines at the top of the file */
		(APTR)estags
	};
#else
	struct EasyStruct req = {
		sizeof(struct EasyStruct),
		0,
		"MacFilesystem Error",
		text,
		"Ok" /* see REQ_ defines at the top of the file */
	};
#endif
	if (IntuitionBase->FirstScreen != NULL)
		return EasyRequestArgs(NULL, (struct EasyStruct *)&req, NULL, args);
	else
		return 0;
}

/*-----------------------------------------------------------------------*/

/*
 *  Debug stuff....
 */
#ifdef __DEBUG__

static const UWORD rawputchar_m68k[] =
{
	0xCD4B, 0x4EAE, 0xFDFC, 0xCD4B, 0x4E75
};

void KPrintFArgs(UBYTE* fmt, ULONG* args )
{
#undef SysBase
	struct ExecBase *SysBase = *(VOLATILE APTR*)4;
#ifdef __MOSINITDEBUG__
	if (SysBase->ex_DebugFlags & EXECDEBUGF_INIT)
#endif
		RawDoFmt( fmt, args, (void(*)(void)) rawputchar_m68k, SysBase );
}

void knprints(UBYTE *name, LONG namelen)
{
	int i;
	UBYTE debugbuff[0xff];
	ULONG _args[1];
	for (i=0; i<namelen; i++)
		debugbuff[i] = name[i];
	debugbuff[namelen] = '\0';
	_args[0] = (ULONG)debugbuff;
	KPrintFArgs("\"%s\"\n", _args);
}

#endif
